@@ -14,9 +14,9 @@ use constant OFFSET_TO_FIRST_HBIN => 0x1000;
 
 sub new {
     my $class = shift;
-    my $filename = shift;
+    my $filename = shift or croak "No filename specified";
 
-    open my $regfile, "<", $filename or croak "Unable to open $filename: $!";
+    open my $regfile, "<", $filename or croak "Unable to open '$filename': $!";
 
     sysread($regfile, my $regf_header, 0x30);
     if (!defined($regf_header) || length($regf_header) != 0x30) {
@@ -29,13 +29,13 @@ sub new {
     }
     
     my $offset_to_first_key = unpack("x36 V", $regf_header);
+    $offset_to_first_key += OFFSET_TO_FIRST_HBIN;
 
-    sysseek($regfile, OFFSET_TO_FIRST_HBIN + $offset_to_first_key, 0);
+    sysseek($regfile, $offset_to_first_key, 0);
     sysread($regfile, my $nk_header, 0x8);
     if (!defined($nk_header) || length($nk_header) != 0x8) {
-        croak "Could not read first key",
-            sprintf(" at offset 0x%x\n",
-                OFFSET_TO_FIRST_HBIN + $offset_to_first_key);
+        croak "Could not read first key at offset ",
+            sprintf("0x%x\n", $offset_to_first_key);
     }
 
     if (substr($nk_header, 4, 4) eq "nk\x2c\x00") {
@@ -46,9 +46,8 @@ sub new {
         return $self;
     }
     else {
-        croak "Did not find root key",
-            sprintf(" at offset 0x%x\n",
-                OFFSET_TO_FIRST_HBIN + $offset_to_first_key);
+        croak "Did not find root key at offset ",
+            sprintf("0x%x\n", $offset_to_first_key);
     }
 }
 
@@ -70,23 +69,32 @@ sub dump_file {
 
     sysseek($regfile, 0, 0);
     sysread($regfile, my $regf_header, 0x30);
+    if (!defined($regf_header) || length($regf_header) != 0x30) {
+        print "end of data\n";
+        return;
+    }
 
-    my $sig = unpack("a4", $regf_header);
-    print "signature = '$sig'\n";
-    
-    my $lwt = unpack("x12a8", $regf_header);
-    print "lwt = ", decode_win32_filetime($lwt), "\n";
+    my ($sig, $timestamp) = unpack("a4x8a8", $regf_header);
+    print "File signature = '$sig'\n";
+    print "Timestamp = ", decode_win32_filetime($timestamp), "\n";
 
-    sysread($regfile, my $fname, 0xd0); # is 0xd0 enough?
-    $fname = unpack("Z*", decode("UCS-2LE", $fname));
-    print "embedded filename = $fname\n";
+    sysread($regfile, my $embedded_filename, 0x40);
+    if (!defined($embedded_filename) || length($embedded_filename) != 0x40) {
+        print "end of data\n";
+        return;
+    }
 
-    my $pause = <>;
+    $embedded_filename = unpack("Z*", decode("UCS-2LE", $embedded_filename));
+    print "Embedded filename = '$embedded_filename'\n";
 
     my $offset_to_hbin = OFFSET_TO_FIRST_HBIN;
     while (1) {
         sysseek($regfile, $offset_to_hbin, 0);
         sysread($regfile, my $hbin_header, 0x20);
+        if (!defined($hbin_header) || length($hbin_header) != 0x20) {
+            print "end of data\n";
+            return;
+        }
 
         # 0x00 dword = 'hbin' signature
         # 0x04 dword = offset from 0x1000 (start of first hbin) to this hbin
@@ -98,13 +106,13 @@ sub dump_file {
 
         # abort if no signature found; could also check if $size_of_hbin == 0?
         if ($sig eq "hbin") {
-            printf "found hbin at 0x%x ", $offset_to_hbin;
+            printf "hbin block @ 0x%x ", $offset_to_hbin;
             printf "offset_from_first_hbin=0x%x size_of_hbin=0x%x\n",
                 $offset_from_first_hbin,
                 $size_of_hbin;
         }
         else {
-            print "no hbin found - finishing...\n";
+            print "no hbin found\n";
             last;
         }
 
@@ -112,36 +120,40 @@ sub dump_file {
         while ($offset < $size_of_hbin) {
             sysseek($regfile, $offset_to_hbin + $offset, 0);
             sysread($regfile, my $header, 8);
+            if (!defined($header) || length($header) != 8) {
+                print "end of data\n";
+                return;
+            }
 
             my ($size, $sig) = unpack("Va2", $header);
 
-            my $status = "positive";
+            my $sign = "positive";
             if ($size > 0x7fffffff) {
-                $status = "negative";
+                $sign = "negative";
                 $size = (0xffffffff - $size) + 1;
             }
 
             print "  ";
             if ($sig eq "nk") {
-                printf "found $sig (size=0x%x, $status) at 0x%x\n",
-                    $size, $offset;
+                printf "%s @ 0x%x size=0x%x, %s\n",
+                    $sig, $offset, $size, $sign;
             }
             elsif ($sig eq "vk") {
-                printf "found $sig (size=0x%x, $status) at 0x%x\n",
-                    $size, $offset;
+                printf "%s @ 0x%x size=0x%x, %s\n",
+                    $sig, $offset, $size, $sign;
             }
             elsif ($sig eq "lh" || $sig eq "lf"
                 || $sig eq "li" || $sig eq "ri") {
-                printf "found $sig (size=0x%x, $status) at 0x%x\n",
-                    $size, $offset;
+                printf "%s @ 0x%x size=0x%x, %s\n",
+                    $sig, $offset, $size, $sign;
             }
             elsif ($sig eq "sk") {
-                printf "found $sig (size=0x%x, $status) at 0x%x\n",
-                    $size, $offset;
+                printf "%s @ 0x%x size=0x%x, %s\n",
+                    $sig, $offset, $size, $sign;
             }
             else {
-                printf "found ?? (size=0x%x, $status) at 0x%x\n",
-                    $size, $offset; 
+                printf "%s @ 0x%x size=0x%x, %s\n",
+                    '??', $offset, $size, $sign;
             }
 
             last if $size == 0;
